body{padding-top:50px;font-family:"Helvetica Neue",Helvetica,Arial,sans-serif;}
.page-subnav,
.pagination-nav {
    width: 100%;
    height: 40px;
    background-color: #EEE;
    background-repeat: repeat-x;
    background-image: -moz-linear-gradient(top, whiteSmoke 0%, #EEE 100%);
    background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%,whiteSmoke), color-stop(100%,#EEE));
    background-image: -webkit-linear-gradient(top, whiteSmoke 0%,#EEE 100%);
    background-image: -ms-linear-gradient(top, whiteSmoke 0%,#EEE 100%);
    background-image: -o-linear-gradient(top, whiteSmoke 0%,#EEE 100%);
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#f5f5f5', endColorstr='#eeeeee',GradientType=0 );
    background-image: linear-gradient(top, whiteSmoke 0%,#EEE 100%);
    border: 1px solid #E5E5E5;
    -webkit-border-radius: 4px;
    -moz-border-radius: 4px;
    border-radius: 4px;
}

.pagination-fixed {
    position: fixed;
    top: 40px;
    left: 0;
    right: 0;
    z-index: 1020;
    border-color: #D5D5D5;
    border-width: 0 0 1px;
    -webkit-border-radius: 0;
    -moz-border-radius: 0;
    border-radius: 0;
    -webkit-box-shadow: inset 0 1px 0 white, 0 1px 5px rgba(0, 0, 0, .1);
    -moz-box-shadow: inset 0 1px 0 #fff, 0 1px 5px rgba(0,0,0,.1);
    box-shadow: inset 0 1px 0 white, 0 1px 5px rgba(0, 0, 0, .1);
    filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.pagination {
    margin: 10px 0;
    margin-top: 0;
    margin-bottom: 0;
}
.pagination-page-info {
    padding: .6em;
    padding-top: 5px;
    padding-bottom: 0;
    padding-left: 0;
    width: 40em;
    margin: .5em;
    margin-left: 0;
    font-size: 12px;
}
.pagination-page-info b {
    color: black;
    background: #6aa6ed;
    padding-left: 2px;
    padding: .1em .25em;
    font-size: 150%;
}
